
#pragma once

//=============================================================================
//
//	Board class
//  
//	9x9 Go board. 
//	Contains two bit boards, each for black and white
//	Note that this class does not keep track of the prisoners. This is because
//	it is the BoardManager class that checks the liberties and captures
//
//=============================================================================

#include "BitBoard9.h"

class Board
{
public:
	// constructor
	Board() 
	{
		Initialize();
	}

	// initialize
	void Initialize()
	{
		// 端を埋めておく
		boards_[0].Clear();
		boards_[0].FillEdge();
		boards_[1].Clear();
		boards_[1].FillEdge();
	}

	// sets a stone onto the board
	void SetStone(Move move)
	{
		boards_[move.color].SetStone(move.point);
		boards_[move.color].Update();
	}

	// remove a stone from the board
	void RemoveStone(Move move)
	{
		boards_[move.color].RemoveStone(move.point);
		boards_[move.color].Update();
	}

	// remove a group of stones from the board
	void RemoveStones(StoneColor color, BitBoard9& stoneBoard)
	{
		boards_[color].RemoveStones(stoneBoard);
	}

	// rotate the board by 90 degrees clockwise
	void Rotate90Clockwise()
	{
		boards_[0].Rotate90Clockwise();
		boards_[1].Rotate90Clockwise();
	}

	// flip the board horizonatally
	void FlipHorizontally()
	{
		boards_[0].FlipHorizontally();
		boards_[1].FlipHorizontally();
	}

	// returns true if all points included in neighbors are occupied
	bool IsSurrounded(BitBoard9& neighbors)
	{
		BitBoard9 libs;
		libs.SetLibertiesOf(neighbors, boards_[0], boards_[1]);
		
		return libs.IsEmptyBoard();
	}

	// gets the logical product of the neighbors and the empty points on the board onto libs
	void GetLiberties(BitBoard9& neighbors, BitBoard9* libs)
	{
		libs->Clear();
		libs->SetLibertiesOf(neighbors, boards_[0], boards_[1]);
	}

	// gets the logical product of the neighbots and the stones of the specified color onto libs
	void GetSurroundingStones(BitBoard9& neighbors, BitBoard9* libs, StoneColor color)
	{
		libs->Clear();
		libs->SetSurroundingStonesOf(neighbors, boards_[color]);
	}

	// gets the logical product of the neighbots and the stones of the specified color onto libs
	void GetSurroundingStones(BitBoard9& neighbors, BitBoard9* libs)
	{
		libs->Clear();
		libs->SetSurroundingStonesOf(neighbors, boards_[BLACK_STONECOLOR], boards_[WHITE_STONECOLOR]);
	}

	// returns true if the is not a stone at that point
	bool IsEmpty(Point pt)
	{
		return boards_[0].IsEmpty(pt) && boards_[1].IsEmpty(pt);
	}

	// returns true if there is not a stone at that point
	bool IsEmpty(unsigned char x, unsigned char y)
	{
		return boards_[0].IsEmpty(x,y) && boards_[1].IsEmpty(x,y);
	}

	// returns true if there is not a stone in the marked area
	bool IsEmpty(BitBoard9 mask)
	{
		return !boards_[0].HasStonesInCommon(mask) && !boards_[1].HasStonesInCommon(mask);
	}

	// writes the liberties around the point to the bitboard
	void GetLibertiesAround(BitBoard9& libBoard, Point pt)
	{
		libBoard.SetLibertiesAround(pt, boards_[0], boards_[1]);
		libBoard.Update();
	}

	// matches a 3x3 pattern to the current board
	bool Match3x3Pattern(unsigned int pattern, Move move)
	{
		return boards_[move.color ^ 1].Match3x3Pattern(pattern, move.point);
	}

	// returns true if both black stone board and the white stone board has at least one stone around the given point
	bool HasBothColorStoneAround(Point pt)
	{
		BitBoard9 pointsAround;
		pointsAround.InitializeWithPointsAround(pt);

		return boards_[0].HasStonesInCommon(pointsAround) && boards_[1].HasStonesInCommon(pointsAround);
	}

	// overrided equal operator
	bool operator ==(Board& other) 
	{
		return boards_[0] == other.boards_[0] && boards_[1] == other.boards_[1];
	}

	// return true if there is a stone of that color at that point
	bool HasStone(Point pt, StoneColor color)
	{
		return !boards_[color].IsEmpty(pt);
	}

	// return true if there exists a black stone at that point
	bool IsBlack(Point pt)
	{
		return !boards_[0].IsEmpty(pt);
	}

	// return true if there exists a white stone at that point
	bool IsWhite(Point pt)
	{
		return !boards_[1].IsEmpty(pt);
	}

#if defined (ENABLE_DEBUG_OUTPUT)
	void ShowBitBoard()
	{
		printf("black:\n");
		boards_[0].Print();

		printf("white:\n");
		boards_[1].Print();
		
		BitBoard9 black = boards_[0];
		black.Rotate90Clockwise();
		BitBoard9 white = boards_[1];
		white.Rotate90Clockwise();

		printf("black:\n");
		black.Print();
		
		black.FlipHorizontally();
		black.Print();

		printf("white:\n");
		white.Print();
		
		white.FlipHorizontally();
		white.Print();
	}
#endif

private:
	BitBoard9 boards_[2];
};