
#pragma once

//=============================================================================
//
//	BoardManager class
//  
//	Manager for the Board class.
//	Keeps the log of the boards, which enables to undo a move
//
//=============================================================================

#include "Board.h"
#include "StoneChain.h"

// Max number of undos possible (i.e., the number of possible moves the ai can make)
#define MAX_UNDOS	1024

class BoardManager
 {
public:
	// constructor
	BoardManager();
	
	// clears the board and all of the logs taken
	void Clear();

	// sets a stone onto the board
	void SetStone(Move move);
	// checks if the chain still has liberties, and if not, the chain is removed from the board
	bool RemoveIfCaptured(StoneChain* chain);

	// sets the libBoard to be a BitBoard containing the liberties around the given point
	void GetLibertiesAround(BitBoard9& libBoard, Point point);

	// returns true if the board is empty at the point
	bool IsEmpty(Point point);
	// returns true if the board is empty at (x,y)
	bool IsEmpty(unsigned char x, unsigned char y);
	// returns true if the board is empty at the all points maked by the given bit board
	bool IsEmpty(BitBoard9 mask);
	// returns true if the last move made violated the ko rule
	bool IsKoViolated();
	// return true if the chain has no more liberties
	bool IsCaptured(StoneChain* chain);
	// returns true if the board as a stone at the specified point with specified color
	bool HasStone(Point pt, StoneColor color);
	// sets libs to the liberties of the chain
	void GetLiberties(StoneChain* chain, BitBoard9* libs);
	// sets surroundings with all neighboring points of the chain that aren't empty
	void GetSurroundingStones(StoneChain* chain, BitBoard9* surrounding);
	// sets surroundings with all neighboring points of the given point that aren't empty
	void GetSurroundingStones(Point pt, BitBoard9* surrounding);
	// returns the current number of prisoners
	int  GetNumPrisoners(StoneColor color);
	// matches the 3x3 board pattern, see BitBoard9.h for details
	bool Match3x3Pattern(unsigned int pattern, Move move);
	// returns true if there is at least one stone of each color around the given point
	bool HasBothColorStoneAround(Point pt);

	// returns the copy of the current board
	Board GetCurrentBoard() { return *currentBoard_; }

	// undo
	void Undo();

#if defined (ENABLE_DEBUG_OUTPUT)
	// debug functions
	void ShowBoard();
	void ShowBitBoard();
	void ShowChain(StoneChain* chain);
#endif

private:
	Board	boardStack_[MAX_UNDOS];		// stacked log of the board
	Board	*currentBoard_;				// the current board

	// prisoner data
	struct PrisonerDatum
	{
		// one int for each color
		unsigned short prisoners_[2];
	};
	PrisonerDatum		prisonerDataStack_[MAX_UNDOS];				// stack log of prisoner data
	PrisonerDatum*		currentPrisonerData_;						// current pointer to the prisoner data
};