
//=============================================================================
//
//	StoneChainManager class
// 
//	Handles the stone chains, and thus the board also. The AIs will communicate
//	with this class to retrieve the board and chain info.
//
//=============================================================================


#pragma once

#include "StaticStoneChain.h"
#include "BoardManager.h"

#include "StoneChainEnumerator.h"

// number of points on the board。9x9 board has 11*11 = 121 points, including its edge. 
#define BOARD_SIZE 128	
// number of maximum stone chains that the manager can handle
#define MAX_STONE_CHAINS 2 * MAX_UNDOS

class StoneChainManager
{
public:
	// constructor
	StoneChainManager();

	// clear all info
	void Clear();

	// check if the board is empty at the point
	bool IsEmpty(Point pt) { return boardManager_.IsEmpty(pt); }
	// check if the board is empty at the masked area
	bool IsEmpty(BitBoard9 mask) { return boardManager_.IsEmpty(mask); }
	// check if the board has a stone at the point
	bool HasStone(Point pt, StoneColor color) { return boardManager_.HasStone(pt, color); }
	// check if the move is legal
	bool IsLegal(Move move);
	// check if the move is legal, regardless of the move color
	bool IsLegalForBothColor(Point point);
	// check if the move is legal, and the stone played cannot be taken in the next turn
	bool IsSafe(Move move);

	// set move
	void SetStone(Move move);
	// undo
	void Undo();


	// checks if a move to capture target exists
	// move is the output, the capturing move is stored iff the return value is true
	bool GetCapturingMove(StoneChain* target, BitBoard9* move);
	// checks if there is at least one chain around the specified move
	bool HasAtariChainAround(Move move);
	// checks if there are at least one of each colored stone around the point
	bool HasBothColorStoneAround(Point pt);
	// returns the current board
	Board GetCurrentBoard() { return boardManager_.GetCurrentBoard(); }

	// get the liberties of the chain
	void GetLiberties(StoneChain* chain, BitBoard9* libs) { boardManager_.GetLiberties(chain, libs); }
	// match 3x3 pattern to the board at move
	bool Match3x3Pattern(unsigned int pattern, Move move) { return boardManager_.Match3x3Pattern(pattern, move); }

	// get the enumerator of chains on the board;
	StoneChainEnumerator GetEnumerator();
	// get the enumerator of opponent neighboring chains of the argument chain
	NeighborStoneChainEnumerator GetNeighborEnumeratorOf(StoneChain* chain);
	// get the enumerator of chains around the given point
	NeighborStoneChainEnumerator GetNeighborEnumeratorAround(Point pt);
	// returns the stone chain that has just been set on the board
	StoneChain* GetNewestStoneChain() { return nextStoneChain_ - 1;}
	// returns the chain at the point
	StoneChain* GetChainAt(Point pt);

#if defined (ENABLE_DEBUG_OUTPUT)
	// print board
	void ShowBoard() { boardManager_.ShowBoard(); }
	void ShowBitBoard() { boardManager_.ShowBitBoard(); }
	// print chains
	void ShowAllChains(char** comments);
#endif

private:
	struct StoneChainState
	{
		StoneChain* chainMap_[BOARD_SIZE];
		bool        chainState_[MAX_STONE_CHAINS];
	};

	StoneChainState		chainStateStack_[MAX_UNDOS];				// undo stack of stone chain states
	StoneChain*			stoneChains_;								// stack of stone chains

	StoneChainState*	currentChainState_;							// current stone chain state, holding memory address to the stone chain on each point on the board 
	StoneChain*			nextStoneChain_;							// the next stone chain to be used

	// place holder for the edge
	StaticStoneChain	edgeChain_;

	// let the board manager take care of the board
	BoardManager	boardManager_;

	// enumerator
	StoneChainEnumerator enumerator_;
};